@@ -5,11 +5,15 @@ use strict;
 use 5.008;
 use Rubyish;
 use HTML::PullParser;
+# use Data::Dump qw(pp);
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
+
+my $indent_offset = 4;
 
 def load($html) {
-    $self->{html} = $html
+    $self->{html} = $html;
+    $self;
 }
 
 def dump {
@@ -26,12 +30,15 @@ def convert {
         text  => '"T", text',
         end   => '"E", tagname',
     );
-    
+
     my $current_tag = "";
     my @stack = ();
+    my $indent = 0;
     while(my $token = $p->get_token) {
+        # warn $token->[0],"\n";;
         if ($token->[0] eq 'S') {
             push @stack, { tag => $token->[1], attr => [@$token[2..$#$token]]};
+            $indent += 1;
         }
         elsif ($token->[0] eq 'T') {
             unless($token->[1] =~ /^\s*$/s ) {
@@ -39,14 +46,19 @@ def convert {
             }
         }
         elsif ($token->[0] eq 'E') {
+            # pp $token;
             my @content;
-
             my $content = pop @stack;
             while (!$content->{tag} || $content->{tag} ne $token->[1]) {
                 push @content, $content;
                 $content = pop @stack;
             }
+
             my $start_tag = $content;
+
+            my $indent_str = " " x ($indent * $indent_offset);
+            my $indent_str2 = " " x ( ($indent + 1) * $indent_offset);
+
             my $attr = "";
             my @attr = @{$start_tag->{attr}};
             if (@attr) {
@@ -54,30 +66,49 @@ def convert {
                     $attr .= qq{ $k => "$v"};
                 }
                 $attr = "($attr )";
-            } 
+            }
 
             if (@content == 1) {
                 my $content_text = $content[0]->{code};
-                $content_text = "\"$content[0]->{text}\"" unless $content_text;
+                if (!$content_text && $content[0]->{text}) {
+                    $content_text = "\"$content[0]->{text}\""
+                }
+                $content_text ||= '';
                 push @stack, {
-                    code => "$start_tag->{tag}${attr} { $content_text }"
+                    code => "\n${indent_str}$start_tag->{tag}${attr} {\n${indent_str2}$content_text\n${indent_str}};\n"
                 };
             }
             else {
-                my $content_code = join "\n", map { $_->{code} || $_->{text} } reverse @content;
+                for (@content) {
+                    if ($_->{text}) {
+                        $_->{code} = "outs \"$_->{text}\";";
+                        $_->{text} = undef;
+                    }
+                }
+                my $content_code = join "\n", map { $_->{code}||"" } reverse @content;
+                # pp $start_tag->{tag}, $start_tag->{indent};
                 push @stack, {
-                    code => "$start_tag->{tag}${attr} { $content_code }"
+                    code => "\n${indent_str}$start_tag->{tag}${attr} {\n${indent_str2}$content_code\n${indent_str}};\n"
                 };
             }
+
+            $indent -= 1;
         }
     }
 
-    my $ret = join "\n", map { $_->{code} || $_->{text} } @stack;
-    $ret = "sub {\n$ret\n}\n";
+    my $ret = join "\n", "sub {", (map { $_->{code} || $_->{text} } @stack), "\n}\n";
+
+    # Squeeze empty lines.
+    $ret =~ s/\n\s*\n/\n/g;
+    $ret =~ s/\{\n\s+\}(;?)\n/{}$1\n/g;
+
+    # Re-org all text only blocks to a single line.
+    $ret =~ s/\{\n\s+(".+")\n\s+\}(;?)\n/{ $1 }$2\n/g;
+
     return $ret;
 }
 
-1; 
+1;
 __END__
 
 =head1 NAME
@@ -114,7 +145,7 @@ This document describes Markapl::FromHTML version 0.01
 
 This module converts HTML to Markapl perl code.
 
-=head1 INTERFACE 
+=head1 INTERFACE
 
 =over
 
